-- Matriz de LEDs multiplexada. 40 LEDs 8 x 5 (linha x coluna)
--
-- Acendimento: colocar a coluna que se quer acender em ni­vel alto e a linha em ni­vel baixo 
--	O controle e feito a partir de dois vetores de saida, um para as colunas, outro para as linhas.
--	O hardware elaborado faz a rotacao conti­nua do vetor das colunas, fazendo com que seja acesa de uma vez.
--	O vetor das linhas e armazenado (no package) e seu valor e atualizado de forma sincronizada com o vetor das colunas,
--	de maneira a formar o caracter de forma multiplexada (uma coluna por vez).
--	
--	A exibiÃ§ao e controlada pelo parametro NUM_MUX, o qual define quantas vezes o mesmo caracter sera multiplexado.
--	
--	O parametro NUM_CHAR define o comprimento da string a ser exibida.
-- 
-- Parametros da mensagem a ser escrita do package matriz_leds_pkg.vhd

library ieee;
	use ieee.std_logic_1164.all;
	use ieee.std_logic_unsigned.all;
	use ieee.numeric_std.all;

library work;
	use work.matriz_leds_pkg.all;  

entity matriz_leds is
generic(
		PRESCALER: integer := 25000
);
port(
		clk_i			:		in std_logic;
		rst_i			:		in std_logic;
		idx_i		: 		in std_logic_vector (4 downto 0);
		col_o	:		out std_logic_vector (4 downto 0);
		row_o	: 		out std_logic_vector (7 downto 0)
);
end matriz_leds;

architecture behavior of matriz_leds is
	signal sg_out_col		: std_logic_vector (4 downto 0);
	signal sg_out_row		: std_logic_vector (7 downto 0);
	signal stop				: std_logic;
	signal clk_en			: std_logic;
	signal idx				: std_logic_vector (4 downto 0);
	
begin

	col_o <= not sg_out_col;  -- sai­das invertidas para se adequar ao funcionamento dos LEDs
	row_o <= not sg_out_row;
	
	idx <= idx_i;--"1010" when (idx_i = "1111" or idx_i > "1010") else idx_i;
	
	process(clk_i,rst_i)
	variable cnt: integer range 0 to PRESCALER := 0;
	begin
		if (rst_i = '1') then
			cnt := 0;
		elsif(rising_edge(clk_i)) then
			if	cnt = PRESCALER then
				clk_en <= '1';
				cnt := 0;
			else
				cnt := cnt + 1;
            clk_en <= '0';
			end if;			
		end if;
	end process;
	
	process(clk_i, rst_i) -- processo que multiplexa um caracter (5 colunas) infinitas vezes
	
	variable col: integer range 0 to 4;				-- i­ndice coluna
	variable mux: integer range 0 to NUM_MUX; 	-- controle de repeticoes de exibicao de cada caracter

	begin
		if(rst_i = '1') then
			col := 0;
			mux := 0;
			sg_out_col <= "00001";
		elsif(rising_edge(clk_i)) then	
         if (clk_en = '1') then
            sg_out_col <= sg_out_col;		-- inicializações padrão
            sg_out_row <= sg_out_row;
            if(mux = NUM_MUX) then 	-- confere se já¡ multiplexou o numero determinado de vezes o mesmo caracter
               mux := 0;
					col := 0;
            else
				if (col = 4) then   	-- incremento do í­ndice (referente à coluna do sí­mbolo)
					col := 0;
					mux := mux+1;   
				else
					col := col+1;
				end if;
				if(sg_out_col = "10000") then	 -- rotacao
					sg_out_col <= "00001";
				else
					sg_out_col <= sg_out_col(3 downto 0) & '0'; -- rotação do vetor que acende as colunas
				end if;
            end if;
         end if;
      
      sg_out_row <= msg(TO_INTEGER(unsigned(idx)))(col); -- referencia a busca do caracter pelo índice da entrada		
		end if;
	end process;
		
end behavior;